﻿//
// <copyright file="Default.aspx.cs" company="Microsoft">
//     Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
namespace Samples.CloudDigitalAssetStorageManagementWorker
{
    using Microsoft.ServiceHosting.ServiceRuntime;
    using Microsoft.Samples.ServiceHosting.StorageClient;
    using System;
    using System.Configuration;
    using System.Collections.Generic;
    using System.Drawing;
    using System.IO;
    using System.Net;
    using System.Linq;
    using System.Threading;
    using System.Text;
    using Samples.CloudDigitalAssetStorageService_Utilities;
    
    // Worker Role class responsible for tagging and thumbnail extraction 
    public class WorkerRole : RoleEntryPoint
    {
        enum enumAssetType { image=0, documents=1, other=2 };
        
        //// Add business logic to be executed by this worker role.
        public override void Start()
        {
            int assetType = 0;
            Stream thumbNailStream = null;
            BlobContents content = null;
            BlobProperties props = null;
            BlobContainer container = null;
            MessageQueue queue = null;
                
            try
            {
                if (RoleManager.IsRoleManagerRunning)
                {
                    RoleManager.WriteToLog("Information", "Worker process is executing ...");
                }
                container = StorageUtility.GetDigitalAssetContainer();
                queue = StorageUtility.GetMetaDataMakerQueue();
                //// Now that the queue and the container have been created in the above initialization process, get messages
                //// from the queue and process them individually.
            }
            catch (Exception e)
            {
                RoleManager.WriteToLog("Error", string.Format("Exception when processing queue item. Message: '{0}'", e.Message));
            }
            //// Continue executing 
            while (true)
                {
                    try
                    {
                        if (queue != null)
                        {
                            Message msg = queue.GetMessage();
                            if (msg != null)
                            {
                                //// Get path to image in the blob store
                                string path = msg.ContentAsString();
                                RoleManager.WriteToLog("Information", string.Format("Mesasge Dequeued from Queue '{0}'", path));
                                /// get metadata about image; verify the content type
                                /// depending on content type execute the following
                                AssetMetadata assetMeataData = StorageUtility.GetMetaData(path);
                                if (assetMeataData != null)
                                {
                                    if (assetMeataData.ContentType != null)
                                    {
                                        string contentType = assetMeataData.ContentType;
                                        RoleManager.WriteToLog("Information", string.Format("Content Type: '{0}'", contentType));
                                        if (contentType.StartsWith("image/"))
                                        {
                                            assetType = (int)enumAssetType.image;
                                        }
                                        else if (contentType.StartsWith("application/"))
                                        {
                                            assetType = (int)enumAssetType.documents;
                                        }
                                        else
                                        {
                                            assetType = (int)enumAssetType.other;
                                        }
                                    }
                                    //// Get Image from the blob
                                    content = new BlobContents(new MemoryStream());
                                    props = container.GetBlob(path, content, false);
                                    //// Now generate meta-data from the asset
                                    //// Here we will generate a thnumb Nail
                                    thumbNailStream = CreateThumbnail(assetType, content.AsStream);
                                    ////Create the thumbnail and store in the blob 
                                    container.CreateBlob(new BlobProperties(Path.Combine("asset-thumbnail", props.Name))
                                    {
                                        ContentType = "image/jpeg"
                                    }, new BlobContents(thumbNailStream), true);
                                    RoleManager.WriteToLog("Information", string.Format("Done with '{0}'", path));
                                }
                                //// Now deque the message from the queue
                                queue.DeleteMessage(msg);
                            }
                        }
                    }
                    catch (StorageException e)
                    {
                        RoleManager.WriteToLog("Error", string.Format("Exception when processing queue item. Message: '{0}'", e.Message));
                    }
                    Thread.Sleep(1000);
                }
        }

        //// Share the health of the application.
        public override RoleStatus GetHealthStatus()
        {
            return RoleStatus.Healthy;
        }

        //// Generate a thumbnail for the input.
        private Stream CreateThumbnail(int assetType, Stream input)
        {
            MemoryStream ms = null;
            switch (assetType)
            {
                case 0:
                    RoleManager.WriteToLog("Information", "Generating Thumbnail for image");
                    ms = (MemoryStream)generateThubNail(input);
                    RoleManager.WriteToLog("Information", "Done generating Thumbnail for image");
                    break;
                case 1:
                case 2:
                    try
                    {
                        //// Use a generic image for docs and other types of assets and generate thumbnail from that.
                        RoleManager.WriteToLog("Information", "Generating Thumbnail for docs/others");
                        System.Reflection.Assembly thisExe;
                        thisExe = System.Reflection.Assembly.GetExecutingAssembly();
                        using (System.IO.Stream file =
                            thisExe.GetManifestResourceStream("CloudDigitalAssetStorageManagementWorker.DocBitmap1.bmp"))
                        {
                            ms = (MemoryStream)generateThubNail(file);
                        }
                        RoleManager.WriteToLog("Information", "Done generating Thumbnail for docs/others");
                    }
                    catch
                    {
                        RoleManager.WriteToLog("Information", "Exception generating Thumbnail for docs/others");
                        throw;
                    }
                    break;
                default:
                    break;
            }
            return ms;
        }
        //// Generate a thumbnail from the input image file.
        private Stream generateThubNail(Stream input)
        {
            MemoryStream ms = new MemoryStream();
            using (Bitmap orig = new Bitmap(input))
            {

                int width;
                int height;
                if (orig.Width > orig.Height)
                {
                    width = 128;
                    height = 128 * orig.Height / orig.Width;
                }
                else
                {
                    height = 128;
                    width = 128 * orig.Width / orig.Height;
                }

                using (Bitmap thumb = new Bitmap(width, height))
                {
                    using (Graphics graphic = Graphics.FromImage(thumb))
                    {
                        graphic.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                        graphic.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
                        graphic.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
                        graphic.DrawImage(orig, 0, 0, width, height);
                        thumb.Save(ms, System.Drawing.Imaging.ImageFormat.Jpeg);
                        ms.Seek(0, SeekOrigin.Begin);
                    }
                }
            }
            return ms;
        }
    }
}
