﻿//
// <copyright file="StorageUtilities.cs" company="Microsoft">
//     Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
namespace Samples.CloudDigitalAssetStorageService_Utilities
{
    using Microsoft.ServiceHosting.ServiceRuntime;
    using System;
    using System.Collections.Generic;
    using System.Collections.Specialized;
    using System.Linq;
    using System.Text;
    using System.IO;
    using System.ServiceModel;
    using System.Runtime.Serialization;
    // Adding a reference to Storage client sample shipped with SDK
    using Microsoft.Samples.ServiceHosting.StorageClient;
    
    public static class StorageConstants
    {
        // blob related constants
        public const string BlobContainerName = "fabrikamassetgallery";
        public const string BlobImageThumbNailPathName = "asset-thumbnail";
        // Queue related constants
        public const string QueueContainerName = "fabrikamimageprocessingqueue";
    }

    public static class StorageUtility
    {
        // utility classes from StorageClient 
        private static BlobStorage blobStorage;
        private static QueueStorage queueStorage;

        private static bool CreatedContainerAndQueue = false;
        private static object Objlock = new Object();

        public static void CreateOnceContainerAndQueue()
        {
            if (CreatedContainerAndQueue)
            {
                return;
            }
            lock (Objlock)
            {
                if (CreatedContainerAndQueue)
                {
                    return;
                }

                try
                {
                    blobStorage = BlobStorage.Create(StorageAccountInfo.GetDefaultBlobStorageAccountFromConfiguration());
                    blobStorage.GetBlobContainer(StorageConstants.BlobContainerName).CreateContainer(null, ContainerAccessControl.Public);

                    queueStorage = QueueStorage.Create(StorageAccountInfo.GetDefaultQueueStorageAccountFromConfiguration());
                    queueStorage.GetQueue(StorageConstants.QueueContainerName).CreateQueue();
                }
                catch (Exception e)
                {
                    RoleManager.WriteToLog("Information", string.Format("Message  '{0}'", e.InnerException));
 
                    System.Text.StringBuilder buffer = new System.Text.StringBuilder();
                    buffer.Append("The Windows Azure storage services cannot be contacted ").
                        Append("via the current Windows Azure account configuration or the local development storage tool is not running. ").
                        Append("Please start the development storage tool if you run the service locally!");

                    throw new System.Exception(buffer.ToString());
                }

                CreatedContainerAndQueue = true;
            }
        }

        public static BlobContainer GetDigitalAssetContainer()
        {
            CreateOnceContainerAndQueue();
            return blobStorage.GetBlobContainer(StorageConstants.BlobContainerName);
        }

        public static MessageQueue GetMetaDataMakerQueue()
        {
            CreateOnceContainerAndQueue();
            return queueStorage.GetQueue(StorageConstants.QueueContainerName);
        }
        public static string StoreImage(String inContentType, Stream inFile)
        {
            BlobProperties props = 
                new BlobProperties(Path.Combine(("assets"), string.Format("{0:10}_{1}", DateTime.Now.Ticks, Guid.NewGuid())))
                {
                    ContentType = inContentType
                };
            StorageUtility.GetDigitalAssetContainer().CreateBlob(props, new BlobContents(inFile), true);
            return props.Name;
        }

        public static void UpdateAssetMetadata(String assetId, AssetMetadata metadata)
        {
            // need to update the blob itself to update the content type
            using (MemoryStream buff = new MemoryStream())
            {
                BlobContents content = new BlobContents(buff);
                BlobProperties props = StorageUtility.GetDigitalAssetContainer().GetBlob(assetId, content, false);
                if (metadata.ContentType != null)
                {
                    props.ContentType = metadata.ContentType;
                }
                if (metadata.Metadata != null)
                {
                    props.Metadata = new NameValueCollection();
                    Dictionary<string, string>.KeyCollection keyColl = metadata.Metadata.Keys;
                    foreach (string key in keyColl)
                    {
                        string val;
                        metadata.Metadata.TryGetValue(key, out val);
                        if (val != null)
                        {
                            props.Metadata.Add(key, val);
                        }
                    }
                }
                bool updateDone = StorageUtility.GetDigitalAssetContainer().UpdateBlobIfNotModified(props, content);
            } 
        }

        public static void SetAssetMetadata(String assetId, AssetMetadata metadata)
        {
            BlobProperties props = StorageUtility.GetDigitalAssetContainer().GetBlobProperties(assetId);
            if (metadata.Metadata != null)
            {
                props.Metadata = new NameValueCollection();
                Dictionary<string, string>.KeyCollection keyColl = metadata.Metadata.Keys;
                foreach (string key in keyColl)
                {
                    string val;
                    metadata.Metadata.TryGetValue(key, out val);
                    if (val != null)
                    {
                        props.Metadata.Add(key, val);
                    }
                }
            }
            StorageUtility.GetDigitalAssetContainer().UpdateBlobMetadata(props);
        }
        
        public static bool ProcessAsset(String assetId)
        {
            StorageUtility.GetMetaDataMakerQueue().PutMessage(new Message(assetId));
            if (RoleManager.IsRoleManagerRunning)
            {
                RoleManager.WriteToLog("Information", string.Format("Asset Enqueued '{0}' for meta-data management", assetId));
            }
            return true;
        }
        
        public static Stream GetAsset(string assetId)
        {
            throw new NotImplementedException();
        }
        public static IEnumerable<object> GetThumpNailImages()
        {
            return StorageUtility.GetDigitalAssetContainer().ListBlobs(StorageConstants.BlobImageThumbNailPathName, false);
        }
        public static AssetMetadata GetMetaData(string assetId)
        {
            AssetMetadata asset = null;
            BlobProperties props = StorageUtility.GetDigitalAssetContainer().GetBlobProperties(assetId);
            if (props !=null){
                    asset = new AssetMetadata();
                    asset.AssetName = props.Name;
                    asset.AssetUri = props.Uri;
                    asset.ThumbNailUri = props.Uri;
                    asset.ContentType = props.ContentType;
                    asset.ContentLength = props.ContentLength;
                    asset.ContentLanguage = props.ContentLanguage;
                    asset.ContentEncoding = props.ContentEncoding;

                    if (props.Metadata != null)
                    {
                        string[] keys = props.Metadata.AllKeys;
                        asset.Metadata = new Dictionary<string, string>();
                        for (int i = 0; i < keys.Count(); i++)
                        {
                            asset.Metadata.Add(keys[i], props.Metadata.Get(i));
                        }
                    }    
            }
            return asset;
        }
        public static IEnumerable<AssetMetadata> GetAllMetaData()
        {
            LinkedList<AssetMetadata> assetList = null;
            IEnumerable<object> allBlobs = StorageUtility.GetDigitalAssetContainer().ListBlobs("assets", false);
            if(allBlobs !=null)
            {
                assetList = new LinkedList<AssetMetadata>();
            }    
            foreach (BlobProperties blob in allBlobs)
            {
                AssetMetadata assetMeataData = StorageUtility.GetMetaData(blob.Name);
                assetList.AddLast(assetMeataData);
            }
         return assetList; 
        }
    }
}